﻿using Microsoft.VisualBasic;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MontoyaTech.MySqlPlus
{
    /// <summary>
    /// The outline of a MySqlSession which contains a MySqlManagedConnection
    /// and a few other help functions.
    /// </summary>
    public class MySqlSession : IDisposable
    {
        /// <summary>
        /// Returns whether or not this MySqlSession has a valid connection.
        /// </summary>
        public bool IsConnected
        {
            get
            {
                try
                {
                    if (this.Connection == null || !this.Connection.IsConnected)
                        return false;
                }
                catch
                {
                    return false;
                }

                return true;
            }
        }

        /// <summary>
        /// The underlying MySql Connection that this Session is using.
        /// </summary>
        public MySqlManagedConnection Connection = null;

        /// <summary>
        /// The raw connection string used to open a MySqlConnection.
        /// </summary>
        protected string ConnectionStr = null;

        /// <summary>
        /// Creates a new MySqlSession with a connection string.
        /// </summary>
        /// <param name="connectionStr"></param>
        public MySqlSession(string connectionStr)
        {
            if (string.IsNullOrWhiteSpace(this.ConnectionStr))
                throw new Exception("Must provide a valid non null or empty MySql connection string.");

            this.ConnectionStr = connectionStr;
        }

        /// <summary>
        /// Attempts to connect to the MySql Server using the connection information.
        /// </summary>
        /// <exception cref="Exception"></exception>
        public void Connect()
        {
            lock (this)
            {
                //Dont connect again if we are already connected.
                if (this.Connection == null)
                {
                    if (!string.IsNullOrWhiteSpace(this.ConnectionStr))
                    {
                        try
                        {
                            //Setup the mysql connection.
                            this.Connection = new MySqlManagedConnection(this.ConnectionStr);

                            //Attempt to open the connection, this will make sure its valid and works for us.
                            this.Connection.Open();
                        }
                        catch (Exception ex)
                        {
                            throw new Exception("Failed to connect to MySql database.", ex);
                        }
                    }
                    else
                    {
                        throw new Exception("Missing connection details to connect to MySql database.");
                    }
                }
            }
        }

        /// <summary>
        /// Disconnects from the MySql Server this Session is connected to.
        /// </summary>
        public void Disconnect()
        {
            lock (this)
            {
                try
                {
                    if (this.Connection != null)
                    {
                        try { this.Connection.Close(); } catch { }

                        try { this.Connection.Dispose(); } catch { }

                        this.Connection = null;
                    }
                }
                catch
                {
                    this.Connection = null;
                }
            }
        }

        ~MySqlSession()
        {
            this.Disconnect();
        }

        public virtual void Dispose()
        {
            this.Disconnect();
        }
    }
}
